/*----------------------Web Analytics-----------------------------------------*/
/* WAPATH                                                                     */
/* Create the Interactive Path Report                                         */
/*                                                                            */
/*    two calls are used: the 1st (&INFO eq YES) creates a template data set  */
/*    that serves as the model for the report data set, which is created      */
/*    by the 2nd call.                                                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED; Kevin Scott, SASKXS                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20040930      Program Created                 saskxs              NA       */
/* 20041215      Remove supportcount and         cabahl             C0001     */
/*               replaced with wab_interactive_support                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    The following macros need to be available:                              */
/*      -- none --                                                            */
/*                                                                            */
/*    The following macro variables need to be defined and assigned values    */
/*       by the User Interface/Middleware                                     */
/*    &INFO:       a switch variable that indicates whether to create a       */
/*                 place-holding (1 obs with missing values) or a populated   */
/*                 (summarization of actual data) version of &OUTPUT_DATASET  */
/*    &START_PAGE: a comma-delimited list of the reference numbers for the    */
/*                 path start pages.  the $IDXTORF format associates the      */
/*                 reference number with the actual REQUESTED_FILE.           */
/*    &END_PAGE:   a comma-delimited list of the reference numbers for the    */
/*                 path end pages.  the $IDXTORF format associates the        */
/*                 reference number with the actual REQUESTED_FILE.  if no    */
/*                 END_PAGEs have been specified, the report includes all     */
/*                 end pages that were associated with any &START_PAGE.       */
/*    &TOPN:       the maximum number of rows that will be in the results     */
/*                 table.                                                     */
/*    &SEQUENCE_LENGTH: the maximum number of requested files in a path.  if  */
/*                 no value has been provided, the default of 7 is used.  if  */
/*                 the provided value is 1 or less, the default of 2 is used. */
/*    &DATES:      a comma-delimited list of the YYYY-MM-DD values that are   */
/*                 the dates for the analysis.                                */
/*    &LIBREFS:    a comma-delimited list of the SAS libnames that need to    */
/*                 be defined by the process.                                 */
/*    &LIBPATHS:   a comma-delimited list of the location that corresponds    */
/*                 to the corresponding libname in the LIBREFS list.          */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*      -- none --                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*      -- none --                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/

%macro wapath;

%******************************************************************************;
%* Initialize SWA resources                                                    ;
%******************************************************************************;
%wautils;
%let   wab_note    = %nrstr(NOTE:(&macname) %left(%sysfunc(time(),time.)));
%let   wab_warning = %nrstr(WARNING:(&macname) %left(%sysfunc(time(),time.)));
%let   wab_error   = %nrstr(ERROR:(&macname) %left(%sysfunc(time(),time.)));
%let   wab_rc      = 0;


%******************************************************************************;
%* Initialize local macro variables to default values.                         ;
%******************************************************************************;
data _null_;
  length tmprefs  $ 1024
         tmppaths $ 4096;

   tmprefs="&librefs";
   tmppaths="&libpaths";

   %* make sure that both lists have the same number of elements    ;

   num_ref_commas=countc(tmprefs,',');
   num_path_commas=countc(tmppaths,',');

   if num_ref_commas eq num_path_commas then do;

      do i=1 to (1+num_ref_commas);
         libname=scan(tmprefs,i,',');
         libpath=strip(scan(tmppaths,i,','));
         rc = filename("fileref",libpath);
         did = dopen("fileref");

         if did then do;
            rc=dclose(did);
            rc = libname(strip(libname), libpath);
            call symput("_wab_lib_error_","0");
         end;

         else do;
            call symput("_wab_lib_error_","2");
            call symput("_wab_lib_msg_"
                       ,"The physical location "||strip(libpath)
                      ||" either does not exist or cannot be "
                      ||"opened. The library "||strip(libname)
                      ||" must be respecified with a valid location.");
         end;

      end;

   end;

   else do;
      call symput("_wab_lib_error_","3");
      call symput("_wab_lib_msg_","You have different numbers of "
                                  ||"library references than physical paths."
                                  ||"Please respecify your libraries.");
   end;

run;

%if %sysfunc(exist(config.waconfig)) ne 0 %then %do;

   data _null_;
      set config.waconfig;
      call symput(parameter_name,strip(value));
   run;

%end;

%else %do;

   %let wab_interactive_support=3;

%end;

%if &sequence_length eq  %then
   %let sequence_length=7;

%if &sequence_length le 1 %then
   %let sequence_length=2;

%let itemvarlist=%cmpres(item1-item%left(&sequence_length));

%******************************************************************************;
%* Create a unique id for the dsn that will contain the pathing template.      ;
%******************************************************************************;
data _null_;
   call symput("_pathing_data_uid_"
              ,compress(
                        translate(   strip(put(datetime(),15.))
                                  || "_"
                                  || strip(put(ranuni(-1),13.11))
                                  ,' '
                                  ,'.'
                                  )
                       )
              );
run;


%if %upcase(&info)=TRUE %then %do;
   %let outdsn=stp.template_&_pathing_data_uid_;

   data &outdsn;
      length &itemvarlist $80;
      format &itemvarlist $80.;
      label
         %do i=1 %to &sequence_length;
            item&i="Item &i"
         %end;;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

   run;

   %goto DONE;
%end;

%else %do;
   %let outdsn=stp.pathing_&_pathing_data_uid_;

   proc format cntlin=summary.page_frequencies;
   run;

   proc dmdb batch data=summary.pathing
                   dmdbcat=stp.dmdbcat_&_pathing_data_uid_
                   out=stp.dmdbdsn_&_pathing_data_uid_
                   normlen=256;

      ID session_id sequence date;
      CLASS reference;
      TARGET reference;
   run;


   %* convert &DATES (YYYYMMDD) to &SASDATES (SAS date values)                 ;
   options noquotelenmax;   /* avoid warning message about unbalanced quotes  */
   data _null_;
      length sas_dates $ 3200;
      date_str="&dates";

      date_cnt=1+countc(date_str,',');

      sas_dates=put(input(scan(date_str,1,','),yymmdd10.),5.);

      do i=2 to date_cnt;
         curr_date=scan(date_str,i,',');
         sas_dates=strip(sas_dates)
                || ','
                || put(input(scan(date_str,i,','),yymmdd10.),5.);
      end;

      call symput('sasdates',strip(sas_dates));

   run;


   proc path dmdbcat=stp.dmdbcat_&_pathing_data_uid_
             data=stp.dmdbdsn_&_pathing_data_uid_
               (keep=session_id sequence date reference
                where=( ( sequence gt 0 )
                         and
                      (  date in (&sasdates)))
               )
             out=stp.pathing_A&_pathing_data_uid_
             support=&wab_interactive_support
             items=&sequence_length;

       customer session_Id;
       sequence sequence;
       target reference;

   run;

   proc means data=stp.pathing_A&_pathing_data_uid_ noprint max;
      var size;
      output out=maxitems max=maxitems;
   run;

   %let maxitems=0;
   data _null_;
      set maxitems;
      call symput("maxitems",strip(put(maxitems,5.)));
   run;

   %if &maxitems eq 0 %then %do;
      data stp.pathing_&_pathing_data_uid_;
         length item1-item&wab_itemset_size $80;
         format item1-item&wab_itemset_size $80.;
         label
         %do i=2 %to &wab_itemset_size;
            item&i="Item &i"
         %end;;
         %wametatr(srce_dsn=paths
                  ,retcode=wab_rc
                  );

         array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;

         item1='PROC PATH could not create results for the request that'
            || ' you specified.';
      run;
      %goto DONE;
   %end;


   %let itemvarlist=%cmpres(item1-item%left(&maxitems));


   data stp.pathing_B&_pathing_data_uid_;
      set stp.pathing_A&_pathing_data_uid_;

      %if &start_page gt  %then %do;
         %PUT **********;
         %PUT START_PAGE=&START_PAGE;
         %PUT **********;
         if input(item1,12.) in (&start_page);
      %end;

      %if &end_page gt  %then %do;
         %PUT END_PAGE  =&END_PAGE;
         %PUT **********;
         %if &maxitems lt 3 %then %do;
            if (size = 2 and input(item2,12.) in (&end_page));
         %end;
         %else %do;
            if (size = 2 and input(item2,12.) in (&end_page))
            %do i=3 %to &maxitems;
               or (size = &i and input(item&i,12.) in (&end_page))
            %end;;
         %end;
      %end;
   run;

   data stp.pathing_&_pathing_data_uid_(drop=item_new rule i);
      length item_new     $ 1024
             &itemvarlist $ 1024;
      format &itemvarlist $80.;
      label
      %do i=1 %to &wab_itemset_size;
         item&i="Item &i"
      %end;;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

      array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;

      set stp.pathing_B&_pathing_data_uid_;

      do i=1 to &wab_itemset_size;
         item_new=put(strip(item{i}),$idxtorf.);
         item{i}=item_new;
      end;

      if _n_ le &topn;

   run;

%end;

%if %get_observation_count
      (indsn=stp.pathing_&_pathing_data_uid_) eq 0 %then %do;
   data stp.pathing_&_pathing_data_uid_;
      length item1-item&wab_itemset_size $80;
      format item1-item&wab_itemset_size $80.;
      label
      %do i=2 %to &wab_itemset_size;
         item&i="Item &i"
      %end;;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

      array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;

      item1='No paths were found for the start and end pages on the date(s)'
         || ' you specified.';
   run;
%end;


%DONE:;
%if %upcase(&info) ne TRUE %then %do;
   proc datasets library=stp nolist;
      delete dmdbdsn_&_pathing_data_uid_  / memtype=data;
      delete pathing_A&_pathing_data_uid_ / memtype=data;
      delete pathing_B&_pathing_data_uid_ / memtype=data;
      delete dmdbcat_&_pathing_data_uid_  / memtype=catalog;
   quit;
%end;


%put *****OUTPUT PARAMETERS*****;
%put stp:output_dataset=&outdsn;
%let     output_dataset=&outdsn;


%mend wapath;
